using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using AjaxControlToolkit;
using System.Reflection;
using System.ComponentModel;
using System.Globalization;
using System.Web.UI.WebControls;

namespace Jeffz.Web
{
	public class ModalUpdateProgess : UpdateProgress
	{
		private static IEnumerable<ScriptReference> extenderReferences = null;

		private string _BackgroundCssClass;

		public string BackgroundCssClass
		{
			get
			{
				return _BackgroundCssClass;
			}
			set
			{
				_BackgroundCssClass = value;
			}
		}

		private bool _DropShadow;

		public bool DropShadow
		{
			get { return _DropShadow; }
			set { _DropShadow = value; }
		}

		private bool _Drag;

		public bool Drag
		{
			get { return _Drag; }
			set { _Drag = value; }
		}

		private int _X = -1;

		public int X
		{
			get { return _X; }
			set { _X = value; }
		}

		private int _Y = -1;

		public int Y
		{
			get { return _Y; }
			set { _Y = value; }
		}

		private string _PopupDragHandleControlID;

		[IDReferenceProperty(typeof(WebControl))]
		public string PopupDragHandleControlID
		{
			get { return _PopupDragHandleControlID; }
			set { _PopupDragHandleControlID = value; }
		}

		protected override IEnumerable<ScriptReference> GetScriptReferences()
		{
			foreach (ScriptReference scriptRef in this.GetBaseScriptReferences())
			{
				yield return scriptRef;
			}

			foreach (ScriptReference scriptRef in this.GetExtenderReferences())
			{
				yield return scriptRef;
			}

			ScriptReference scriptReference = new ScriptReference();
			scriptReference.Assembly = typeof(ModalUpdateProgess).Assembly.FullName;
			scriptReference.Name = "Jeffz.Web.ModalUpdateProgress.js";
			yield return scriptReference;
		}

		protected IEnumerable<ScriptReference> GetBaseScriptReferences()
		{
			return base.GetScriptReferences();
		}

		private IEnumerable<ScriptReference> GetExtenderReferences()
		{
			if (extenderReferences == null)
			{
			    lock (typeof(ModalUpdateProgess))
			    {
			        if (extenderReferences == null)
			        {
						extenderReferences = (new ModalPopupExtender() as IExtenderControl).GetScriptReferences();
			        }
			    }
			}
			
			return extenderReferences;
		}

		[Browsable(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new bool DynamicLayout
		{
			get
			{
				return base.DynamicLayout;
			}
			set
			{
				base.DynamicLayout = value;
			}
		}

		protected override IEnumerable<ScriptDescriptor> GetScriptDescriptors()
		{
			if (Page != null && ScriptManager.GetCurrent(this.Page).SupportsPartialRendering && this.Visible)
			{
				ScriptControlDescriptor desc = new ScriptControlDescriptor("Jeffz.Web.ModalUpdateProgress", this.ClientID);
				string updatePanelClientID = null;

				if (!String.IsNullOrEmpty(this.AssociatedUpdatePanelID))
				{
					UpdatePanel updatePanel = this.FindTargetControl(
						AssociatedUpdatePanelID, this, true) as UpdatePanel;

					if (updatePanel != null)
					{
						updatePanelClientID = updatePanel.ClientID;
					}
				}
				
				desc.AddProperty("associatedUpdatePanelId", updatePanelClientID);
				desc.AddProperty("displayAfter", this.DisplayAfter);
				desc.AddProperty("backgroundCssClass", this.BackgroundCssClass);
				desc.AddProperty("dropShadow", this.DropShadow);
				desc.AddProperty("drag", this.Drag);
				desc.AddProperty("x", this.X);
				desc.AddProperty("y", this.Y);

				if (this.PopupDragHandleControlID != null)
				{
					Control popupHandle = this.FindControl(this.PopupDragHandleControlID);
					if (popupHandle != null)
					{
						desc.AddProperty("popupDragHandleControlID", popupHandle.ClientID);
					}
				}				

				yield return desc;
			}

			yield break;
		}

		private Control FindTargetControl(string controlID, Control control, bool searchNamingContainers)
		{
            Control foundControl;

            if (searchNamingContainers)
			{
                Control currentContainer = control;
                foundControl = null;
                while (foundControl == null && currentContainer != control.Page)
				{
                    currentContainer = currentContainer.NamingContainer;
                    if (currentContainer == null)
					{
                        break;
                    }
                    foundControl = currentContainer.FindControl(controlID);
                }
            }
            else
			{
                foundControl = control.FindControl(controlID);
            }
            return foundControl;
        }

	}
}
